//
//inclusion des entetes
//
#include "libarp.h"
#include <pthread.h>
#include <time.h>


//
//declaration des structures
//
typedef struct arp_record
{
   char ip[17];
   char mac[18];
} arp_field;


struct arp_map
{
   char *device;
   int tab_length;
   char requested_ip[17]; 
   arp_field arp_tab[50];
};



//
//les prototypes
//
void *arplisten(void *data);
int ifvrfy(char *intf);
void usage(void);




//
//fonction principale
//
int main(int argc, char **argv)
{
   //
   //declaration des variables locales
   //
   char *arg_device, *arg_target, *arg_command;
   char errbuf[PCAP_ERRBUF_SIZE], rep, **parse_command;
   char *cache_hw_addr;
   struct arp_info *send_info;
   struct libnet_link_int *link;
   struct arp_map map;
   int i, c, mode_single, mode_command, comp, cache_query;
   pthread_t arp_listen;
   pthread_attr_t attr; 
   
   //
   //saisie des options de ligne de commande
   //
   mode_single = 0;
   mode_command = 0;
   arg_device = pcap_lookupdev(errbuf);
   arg_target = NULL;
   while((c = getopt(argc, argv, "sc:d:t:")) != -1)
   {
      switch(c)
      {
         case 's':
	           mode_single = 1;
	           break;
	 case 'c':
	           mode_command = 1;
	           arg_command = optarg;
	           break;
	 case 'd':
	           arg_device = optarg;
	           break;
	 case 't':
	           arg_target = optarg;
	           break;
	 default:
	 	  usage();
      }
   }  
   if (mode_single == 1 && mode_command == 1) usage();
   if (arg_target == NULL) usage();
   if ( arg_device == NULL )
   {
      die("no default network device found");
   }
   if ( ifvrfy(arg_device) != 0)
   {
      printf("interface %s do not exist\n",arg_device);
      usage(); 
   }
   
   
   //
   //interrogation du cache
   //
   cache_query = arp_cache_lookup(arg_device, arg_target, &(cache_hw_addr));
   
   //
   //initialisation du thread d'ecoute reseau
   //
   map.device = (char*)malloc(sizeof(arg_device)+1);
   strncpy(map.device, arg_device, sizeof(arg_device)+1);
   strncpy(map.requested_ip, arg_target, sizeof(map.requested_ip));
   pthread_attr_init(&attr);
   pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
   pthread_create(&arp_listen, &attr, arplisten, (void *) &map);
   printf("use network device: %s\n",map.device );
   
   //
   //envoi des paquets ARP et lancement du thread d'ecoute
   //
   link = libnet_open_link_interface(arg_device, errbuf);
   if ( !link ) die(errbuf);
    
   send_info = init_arp_request_packet(arg_target, arg_device, link);
   //pthread_create(&arp_listen, NULL, arplisten, (void *) &map);
   
   for (i=0; i<6; i++)
   {
      send_arp_packet(link, send_info, arg_device);
   }
   
   
   //
   //Detection de l'ARP spoofing
   //
   pthread_join(arp_listen, NULL);
   if ( map.tab_length == 0 )
   {
      rep = 'y';
      printf("No ARP REPLY. Continue([y]/n)?\n");
      rep = getchar();
      if (rep != 'y') exit(1); 
   }
   else
   {
      i=1;
      
      while (i < map.tab_length)
      {
         comp = strncmp(map.arp_tab[i].mac, map.arp_tab[0].mac, 18);
         if ( comp != 0 )
	     {
                printf("receiving multiple ARP reply: SPOOF detected !!!\n");
	        printf("dangerous MAC->%s, %s\n",map.arp_tab[i].mac,map.arp_tab[0].mac);
	        //break;
           exit(0); 
         }
         i++; 
      }
      comp = strncmp(cache_hw_addr, map.arp_tab[0].mac, 18);
      if ( comp != 0 && cache_query == 0)
      {
            printf("incoherent ARP cache status: SPOOF detected !!!\n");
	    printf("dangerous MAC->%s, %s\n",cache_hw_addr,map.arp_tab[0].mac);
        exit(0);
      }
   }
   
   //
   //Execution de la commande utilisateur
   //   
   if (mode_command == 1)
   {
      parse_command = (char**)malloc(sizeof(char*));
      parse_command[0] = strtok(arg_command, " ");
      i = 0;
      while(parse_command[i] != NULL)
      {
         parse_command[i+1] = strtok(NULL, " ");
	 i++;
      }
      execvp(parse_command[0], parse_command);
   }  
 
   return(0);
}



//
//les routines
////fonction affichant les paramtres de ligne de commande
//
void usage(void)
{
        fprintf(stderr,"Version: 1.0\n"
                "Usage: arpwrap -s [-d interface] -t target\n"
		"       arpwrap -c \"command\" [-d interface] -t target\n"
		"\n"
		"arpwrap has two modes: single (-s) and command (-c).\n"
		"Use the single mode to test an IP you suspect to be spoofed.\n"
		"Use the command mode to secure a command execution.\n"
		"If the command's target is an external host, you must\n"
		"indicate your gateway as arpwrap target.\n\n");
        exit(1);
}


//fonction retournant l'ensemble des interfaces rseau
//
int ifvrfy(char *intf)
{
   int fd, i, comp;
   char buff[1024];
   struct ifconf ifc;
   struct ifreq *ifr;
   
   ifc.ifc_len = sizeof(buff);
   ifc.ifc_buf = buff;
   fd = socket(AF_INET, SOCK_DGRAM, 0);
   if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) 
   {
      fprintf(stderr, "SIOCGIFCONF: %s\n", strerror(errno));
      return (-1);
   }
   close(fd);
   
   ifr = ifc.ifc_req;
   comp = -1;
   for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++) 
   {     
      comp = strncmp(ifr->ifr_name, intf, sizeof(ifr->ifr_name));
      if (comp == 0)
      {
         return(comp);
      }  
   }
   return(comp);
}



//fonction dclenche dans le thread (ecoute et stockage des reponses ARP)
//
void *arplisten(void *data)
{
   int eq,i;
   time_t tempo, max;
   pcap_t *descr;
   char errbuf[PCAP_ERRBUF_SIZE];
   struct arp_info *receive_info;
   struct arp_map *arptab =(struct arp_map*)data;
   
   i = 0;
   receive_info = NULL;
   descr = pcap_open_live(arptab->device, BUFSIZ, 0, 5, errbuf);
   
    
   tempo = time(NULL);
   max = tempo + 3;
   if ( descr == NULL )
   {
      printf("pcap_open_live(): %s\n", errbuf);
      exit(1);
   }
   
   while (tempo < max)
   {
       tempo = time(NULL);
       receive_arp_packet(descr, &receive_info);
       
       if( receive_info != NULL  && receive_info->op_type == ARP_RESPONSE )
       {
          //printf ("arp reply from %s : %s\n", receive_info->ip_addr_sender,receive_info->hw_addr_sender);
          eq = strncmp(receive_info->ip_addr_sender, arptab->requested_ip, sizeof(receive_info->ip_addr_sender));
	  if(eq == 0 && i<50)
	  { 
	     strncpy(arptab->arp_tab[i].ip,receive_info->ip_addr_sender , sizeof(arptab->arp_tab[i].ip));
	     strncpy(arptab->arp_tab[i].mac, receive_info->hw_addr_sender, sizeof(arptab->arp_tab[i].mac));
	     i++;
	  }
	  free(receive_info);
	  receive_info = NULL;
       }
   }
   arptab->tab_length = i;
   pcap_close(descr);
   return((void*)0);
}
