#include "libarp.h"

//
//fonctions exportes
//

struct arp_info* init_arp_request_packet(char *ip, char *dev, struct libnet_link_int *link)
{
   struct arp_info *info;
   struct ether_addr *local_hw_addr;
   u_long local_ip_addr;
   char *hwaddr;
   struct in_addr ipad;
   //char *ipaddr;
   char errbuf[PCAP_ERRBUF_SIZE];
   
   
   local_hw_addr = libnet_get_hwaddr(link, dev, errbuf);
   if (local_hw_addr == NULL) die("cannot determine MAC address");
   local_ip_addr = libnet_get_ipaddr(link, dev, errbuf);
   if (local_ip_addr == 0) die("cannot determine ip address");
   
   hwaddr = (char*)malloc(18);
   //ipaddr = (char*)malloc(16);
   
   ipad.s_addr = betole(local_ip_addr);
   strncpy(hwaddr, get_hw_addr(local_hw_addr->ether_addr_octet), 18);
   
   info = (struct arp_info*)malloc(sizeof(struct arp_info));
   info->op_type = ARP_REQUEST;
   strncpy(info-> hw_addr_sender, hwaddr , 18);
   strncpy(info->hw_addr_rcpt, "ff:ff:ff:ff:ff:ff", 18);
   strncpy(info-> ip_addr_sender, inet_ntoa(ipad) , 16);
   strncpy(info-> ip_addr_rcpt, ip, 16);
   
   return (info);
}

int send_arp_packet(struct libnet_link_int *l , struct arp_info *pkt, char *dev)
{
   int n;
   u_char *buf;
   u_char *device;
   u_char *hw_addr_sender;
   u_char *hw_addr_rcpt;
   u_char *ip_addr_sender;
   u_char *ip_addr_rcpt;
   in_addr_t binip;
   in_addr_t binip2;
   
   device = (u_char*) dev;
   hw_addr_sender = (u_char*)malloc(6);
   hw_addr_rcpt = (u_char*)malloc(6);
   ip_addr_sender = (u_char*)malloc(4);
   ip_addr_rcpt = (u_char*)malloc(4);
   
    if ( pkt->op_type == ARP_REQUEST )
    {
        put_hw_addr(hw_addr_sender, pkt->hw_addr_sender);
	put_hw_addr(hw_addr_rcpt, pkt->hw_addr_rcpt);
	//put_ip_addr(&ip_addr_sender, pkt->ip_addr_sender);
	//put_ip_addr(&ip_addr_rcpt, pkt->ip_addr_rcpt);
	binip = inet_addr(pkt->ip_addr_sender);
	binip2 = inet_addr(pkt->ip_addr_rcpt);
	ip_addr_sender = (u_char*) &binip;
	ip_addr_rcpt = (u_char*) &binip2;
	
	if ( libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1 )
	{
	   perror("libnet_init_packet memory:");
	   exit(1);
	}
	libnet_build_ethernet(hw_addr_rcpt, hw_addr_sender, ETHERTYPE_ARP,NULL,0, buf);
        libnet_build_arp(ARPHRD_ETHER, ETHERTYPE_IP, 6, 4, ARPOP_REQUEST,hw_addr_sender , ip_addr_sender, hw_addr_rcpt,ip_addr_rcpt, NULL, 0, buf + LIBNET_ETH_H);
	
	n = libnet_write_link_layer(l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
	if (n == -1) printf("Packet not sent\n");   
	libnet_destroy_packet(&buf);
    }
    
    if ( pkt->op_type == ARP_RESPONSE )
    {
        put_hw_addr(hw_addr_sender, pkt->hw_addr_sender);
	put_hw_addr(hw_addr_rcpt, pkt->hw_addr_rcpt);
	put_ip_addr(&ip_addr_sender, pkt->ip_addr_sender);
	put_ip_addr(&ip_addr_rcpt, pkt->ip_addr_rcpt);
	
	if ( libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1 )
	{
	   perror("libnet_init_packet memory:");
	   exit(1);
	}
	
        libnet_build_arp(ARPHRD_ETHER, ETHERTYPE_IP, 6, 4, ARPOP_REPLY,hw_addr_sender , ip_addr_sender, hw_addr_rcpt,ip_addr_rcpt, NULL, 0, buf + LIBNET_ETH_H);
	
	n = libnet_write_link_layer(l, dev, buf, LIBNET_ARP_H + LIBNET_ETH_H);    
	libnet_destroy_packet(&buf);
    }
    return (n);  
}




void receive_arp_packet(pcap_t *descr, struct arp_info **info)
{
   const u_char *packet;
   struct pcap_pkthdr hdr;
   struct ether_header *eptr;
   struct arp_packet *arppkt;
   
   packet = pcap_next(descr, &hdr);
   if ( packet == NULL ) die("null packet\n");
   eptr = (struct ether_header*) packet;
   
   if ( ntohs(eptr->ether_type) == ETHERTYPE_ARP )
   {
      arppkt = (struct arp_packet*) packet;
      *info = (struct arp_info*)malloc(sizeof(struct arp_info));
      (*info)->op_type = ntohs(arppkt->op);
      strncpy((*info)->hw_addr_sender,get_hw_addr(arppkt->sndr_hw_addr), 18); 
      strncpy((*info)->hw_addr_rcpt,get_hw_addr(arppkt->rcpt_hw_addr), 18);
      strncpy((*info)->ip_addr_sender,get_ip_addr(arppkt->sndr_ip_addr), 16);
      strncpy((*info)->ip_addr_rcpt,get_ip_addr(arppkt->rcpt_ip_addr), 16);      
   }
  
}


//
//routines
//
void put_hw_addr(u_char* rcpt, char* mac)
{
      int i;
      char c,val = 0;

   for(i=0;i<ETH_HW_ADDR_LEN;i++)
   {
      if( !(c = tolower((unsigned int)(*mac++)))) die("Invalid hardware address");
      if(isdigit((unsigned int)c)) val = c-'0';
      else if(c >= 'a' && c <= 'f') val = c-'a'+10;
      else die("Invalid hardware address");

      *rcpt = val << 4;
      if( !(c = tolower(*mac++))) die("Invalid hardware address");
      if(isdigit((unsigned int)c)) val = c-'0';
      else if(c >= 'a' && c <= 'f') val = c-'a'+10;
      else die("Invalid hardware address");

      *rcpt++ |= val;

      if(*mac == ':')mac++;
    }
}


void put_ip_addr(u_char **bin_ip, char *ip)
{
   struct in_addr addr;
   
   addr.s_addr = inet_addr(ip);
   if (addr.s_addr == -1)
   {
      die("Unknown host or invalid ip address");
   }
   *bin_ip = (u_char *) &(addr.s_addr); 
}


char* get_ip_addr(u_char* ip)
{
   struct in_addr addr;
   memcpy(&addr, ip, sizeof(struct in_addr));
   return ( inet_ntoa(addr) );
}

char* get_hw_addr(u_char* rcpt)
{
  int i,j;
   char *str;
   char hexatab[16]={'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
   j=0;
   str = (char *)malloc(18);
   
   for (i=0; i<ETH_HW_ADDR_LEN; i++)
   {
     str[j]=hexatab[rcpt[i] >> 4];
     str[j+1]=hexatab[rcpt[i]&0x0F];
     if (i < 5) str[j+2]=':';
     j = j + 3;
   }
   str[17] = '\0';
   return(str);
}



int arp_cache_lookup(char *device,char *ipaddr, char **etheraddr)
{
    int fd;
    struct arpreq ar;
    struct sockaddr_in *sin;
    struct ether_addr *ether;
    unsigned long ip;
    
    *etheraddr = (char*)malloc(18);
    ether = (struct ether_addr *)malloc(sizeof(struct ether_addr));
    ip = inet_addr(ipaddr);
    memset((char *)&ar,0,sizeof(ar));
    //strncpy(ar.arp_dev, device, sizeof(ar.arp_dev)); //solaris
    sin = (struct sockaddr_in *)&ar.arp_pa;
    sin->sin_family = AF_INET;
    sin->sin_addr.s_addr = ip;
    if((fd=socket(AF_INET,SOCK_DGRAM,0))<0)
        return -1;
    if(ioctl(fd,SIOCGARP,(caddr_t)&ar)==-1)
    {
        close(fd);
        return -2;
    }
    close(fd);
    memcpy(ether->ether_addr_octet,ar.arp_ha.sa_data,ETH_HW_ADDR_LEN);
    strncpy(*etheraddr, get_hw_addr(ether->ether_addr_octet),18);
    return 0;
}



void die(const char* msg)
{
   fprintf(stderr,"%s\n",msg);
   exit(1);
}

u_long betole(u_long val)
{
   u_long ret;
   //const u_long mask1 = 0x000000ff;
   const u_long mask2 = 0x0000ff00;
   const u_long mask3 = 0x00ff0000;
   const u_long mask4 = 0xff000000;
   
   ret = val << 24;
   ret += (val & mask2) << 8;
   ret += (val & mask3) >> 8;
   ret += (val & mask4) >> 24;
   
   return (ret);
}
